from subprocess import Popen
from time import strftime
import tkinter.ttk
from tkinter.messagebox import showerror
import traceback

from serial.tools.list_ports import comports

from . import usb_ids


def main():
    ports = comports()

    top = tkinter.Tk()
    top.title('COM Ports')

    ports_listbox = tkinter.Listbox(top, width=150, height=15, exportselection=False)
    ports_listbox.pack(fill=tkinter.BOTH, expand=1)

    for i in ports:
        ports_listbox.insert(tkinter.END, '{}: {} [{} {}]'.format(i.device, i.description, *usb_ids.get(i.vid or 0, i.pid or 0)))

    ports_listbox.select_set(0, 0)

    baud_rate_combobox = tkinter.ttk.Combobox(top)
    baud_rate_combobox.pack(side=tkinter.LEFT)

    baud_rate_combobox['values'] = list(map(str, (110, 300, 600, 1200, 2400, 4800, 9600, 14400, 19200, 38400, 57600, 115200, 128000, 256000)))
    baud_rate_combobox.set('115200')

    ports_listbox.focus()

    def ok(_event=None):
        try:
            sel = ports_listbox.curselection()
            print(sel)
            if sel:
                port = ports[sel[0]].device
                baud_rate = baud_rate_combobox.get()
                time_str = strftime('%Y%m%d_%H%M%S')
                logfile = r'C:\Temp\putty_{}_{}_{}.log'.format(time_str, port, baud_rate)
                cmd = ['putty', '-serial', port, '-sercfg', baud_rate + ',8,n,1,N', '-sessionlog', logfile]

                print(' '.join(cmd))
                Popen(cmd)
        except Exception:
            showerror('Error opening PuTTY!', traceback.format_exc())

        top.destroy()

    def cancel(_event=None):
        top.destroy()

    ports_listbox.bind('<Double-1>', ok)
    top.bind('<Return>', ok)
    top.bind('<Escape>', cancel)

    top.focus_force()
    top.mainloop()


if __name__ == '__main__':
    main()



import os
import re
import locale
import shutil
from datetime import datetime
from urllib import request


DEFAULT_PATH = os.path.join(os.path.dirname(__file__), 'usb.ids')
DATE_PAT = re.compile(r'^# Date\: +(\d{4}\-\d{2}\-\d{2} \d{2}\:\d{2}\:\d{2})$')
PAT = re.compile(r'^(?:(?P<vendor_id>[\da-f]{4})|\t(?P<product_id>[\da-f]{4}))  (?P<name>.+)$')
_cache = None


def get_ids_file_date(filename=DEFAULT_PATH):
    if not os.path.isfile(filename):
        return None
    with open(filename) as in_f:
        for line in in_f:
            m = DATE_PAT.findall(line)
            if m:
                return datetime.strptime(m[0], '%Y-%m-%d %H:%M:%S')


def auto_update_ids_file(filename=DEFAULT_PATH):
    file_date = get_ids_file_date(filename)
    req = request.urlopen('http://www.linux-usb.org/usb.ids')

    if file_date is not None:
        locale.setlocale(locale.LC_TIME, 'C')
        update_date = datetime.strptime(req.headers['Last-Modified'], '%a, %d %b %Y %H:%M:%S %Z')

        if file_date >= update_date:
            return False

    with open(filename, 'wb') as f:
        shutil.copyfileobj(req, f)

    return True


def _norm(value, do=True):
    value = value or 0
    if (not do) or isinstance(value, int):
        return value
    return int(value, 16)


def parse_ids_file(filename=DEFAULT_PATH, normalize=True, auto_update=True):
    if auto_update:
        try:
            auto_update_ids_file(filename)
        except Exception:
            pass

    ret = {}
    vendor_id = None
    with open(filename) as in_f:
        for line in in_f:
            match = PAT.match(line)
            if not match:
                continue
            d = match.groupdict()
            if d['vendor_id']:
                vendor_id = _norm(d['vendor_id'], normalize)
                ret[vendor_id] = (d['name'], {})
                continue
            ret[vendor_id][1][_norm(d['product_id'], normalize)] = d['name']
    return ret


def get(vendor_id, product_id=None, auto_update=True):
    global _cache
    if not _cache:
        _cache = parse_ids_file(auto_update=auto_update)
    vendor = _cache.get(_norm(vendor_id), ('', {}))
    if product_id is None:
        return vendor[0]
    return (vendor[0], vendor[1].get(_norm(product_id), ''))


def main():
    import json
    import sys
    args = [int(x, 0) for x in sys.argv[1:]]
    if len(args) >= 2:
        print(' : '.join(get(*args)))
    elif len(args) >= 1:
        print(get(*args))
    else:
        print(json.dumps(parse_ids_file(normalize=False),
                         separators=(',', ': '),
                         indent=2,
                         sort_keys=True))


if __name__ == '__main__':
    main()



from setuptools import setup


setup(
    name='comports',
    version='0.1.0',
    url='https://github.com/zeevro/comports',
    download_url='https://github.com/zeevro/comports/archive/master.zip',
    author='Zeev Rotshtein',
    author_email='zeevro@gmail.com',
    maintainer='Zeev Rotshtein',
    maintainer_email='zeevro@gmail.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: Public Domain',
        'Natural Language :: English',
        'Programming Language :: Python',
    ],
    license=None,
    description='A small GUI for launching PuTTY for COM ports',
    keywords=[
        'PuTTY',
        'COM',
        'serial',
        'GUI',
    ],
    zip_safe=True,
    packages=[
        'comports',
    ],
    install_requires=[
        'pyserial',
    ],
    entry_points=dict(
        gui_scripts=[
            'comports = comports.gui:main',
        ],
    ),
)
